#!/usr/bin/env wish

# This script searches the XML-MAP-file specified by infile
# and creates the MAP-Input-file used by modgen from it. The 
# created file is saved at the location specified by outfile.
proc arima_to_modgen {infile outfile} {
	set inf [open $infile r]
	set content [read $inf]
	close $inf

	# Parse XML-File to a list
	set parsedFile [xml2list $content]
	if {[lindex $parsedFile 0] != "profido"} {
		puts "Input file is not a ProFiDo XML description!"
		exit
	} else {
		if {[lindex [lindex [lindex $parsedFile 2] 0] 0] != "arima"} {
			puts "The File has no Arima Description"
			exit
		} else {
			set arimaTag [lindex [lindex [lindex $parsedFile 2] 0] 2]
			set arcount ""
			set ar ""
			set macount ""
			set ma ""
			set d ""
			set mean ""
			set variance ""
			
			foreach curTag $arimaTag {
				if {[lindex $curTag 0] == "arcount"} {
					set arcount  [lindex [lindex [lindex $curTag 2] 0] 1]
				} else {
					if {[lindex $curTag 0] == "ar"} {
						set ar  [lindex [lindex [lindex $curTag 2] 0] 1]
					} else {
						if {[lindex $curTag 0] == "macount"} {
							set macount  [lindex [lindex [lindex $curTag 2] 0] 1]
						} else {
							if {[lindex $curTag 0] == "ma"} {
								set ma  [lindex [lindex [lindex $curTag 2] 0] 1]
							} else {
								if {[lindex $curTag 0] == "d"} {
									set d  [lindex [lindex [lindex $curTag 2] 0] 1]
								} else {
									if {[lindex $curTag 0] == "mean"} {
										set mean  [lindex [lindex [lindex $curTag 2] 0] 1]
									} else {
										if {[lindex $curTag 0] == "variance"} {
											set variance  [lindex [lindex [lindex $curTag 2] 0] 1]
										}
									}
								}
							}
						}
					}
				}
			}

			# Überprüfen ob wir gültige Ergebnisse bekommen haben

			if {($arcount!="") && ($ar!="") && ($macount!="") && ($ma!="") && ($d!="") && ($mean!="") && ($variance!="")} {
				set outf [open $outfile w+]
				puts $outf "model = f-arima"
				puts $outf "order = $arcount $d $macount"
				puts $outf "ar = $ar"
				puts $outf "ma = $ma"
				puts $outf "mean = $mean"
				puts $outf "variance = $variance"
				close $outf
			} else {
				puts "Fehler! Datei $infile enthält keine gültige Information "
			}
		}
				
	}
}


# This methode parses an XML-File into a tcl-list. 
# This code was copied from the TCL-WiKi (http://wiki.tcl.tk/
 proc xml2list xml {
     regsub -all {>\s*<} [string trim $xml " \n\t<>"] "\} \{" xml
     set xml [string map {> "\} \{#text \{" < "\}\} \{"}  $xml]

     set res ""   ;# string to collect the result
     set stack {} ;# track open tags
     set rest {}
     foreach item "{$xml}" {
	## Ignore XML-comments
	 if {[string range $item 0 0] != "?"} {
		switch -regexp -- $item {
		^# {append res "{[lrange $item 0 end]} " ; #text item}
		^/ {
			regexp {/(.+)} $item -> tagname ;# end tag
			set expected [lindex $stack end]
			if {$tagname!=$expected} {error "$item != $expected"}
			set stack [lrange $stack 0 end-1]
			append res "\}\} "
		}
		/$ { # singleton - start and end in one <> group
		regexp {([^ ]+)( (.+))?/$} $item -> tagname - rest
		set rest [lrange [string map {= " "} $rest] 0 end]
		append res "{$tagname [list $rest] {}} "
		}
		default {
		set tagname [lindex $item 0] ;# start tag
		set rest [lrange [string map {= " "} $item] 1 end]
		lappend stack $tagname
		append res "\{$tagname [list $rest] \{"
		}
		}
		if {[llength $rest]%2} {error "att's not paired: $rest"}
		}
	}


     if [llength $stack] {error "unresolved: $stack"}
     string map {"\} \}" "\}\}"} [lindex $res 0]
 }


##################################################################################
##################################################################################
##################################################################################

# This script parses the XML-MAP-file specified by its first parameter
# and creates the MAP-Input-file used by modgen from it. The created
# file is saved at the location specified by the second parameter.
set infile ""
set outfile ""
set infile [lindex $argv 0]
set outfile [lindex $argv 1]
arima_to_modgen $infile $outfile
exit



