#!/usr/bin/env wish

set os $tcl_platform(platform)

# This Methods checks if the tag-value of the passed node
# equals "ref". If so, the content of the node is returned.
proc get_ref_tag_value {fileTag} {


	if {[lindex $fileTag 0] == "ref"} {
		return [lindex [lindex [lindex $fileTag 2] 0] 1]	
	} else {
		return ""
	}
}

# This script searches the XML-File specified by infile for 
# a <ref>-Tag within a <file>-Tag. If one is found, a link 
# pointing to the referenced file is created at the location 
# specified by outfile. 
proc create_link_to_ref_tag_value {infile outfile} {
	global os
	set inf [open $infile r]
	set content [read $inf]
	close $inf

	# Parse XML-File to a list
	set parsed [xml2list $content]
	if {[lindex $parsed 0] != "profido"} {
		puts "Input file is not a ProFiDo XML description!"
		exit
	}
	set parsed [lindex [lindex $parsed 2] 0]

	if {[lindex $parsed 0] != "file"} {
		puts "Error: File $infile does not contain file informations."
	} else {
		set fileTag [lindex $parsed 2]

		foreach curTag $fileTag {
			if {[set resultString [get_ref_tag_value $curTag]] != ""} {
				break;
			} 
 	   	}
 	   	
		
		if {($resultString != "")} {
 		   	if {[file exists $outfile] == 1} {
				file delete -force $outfile
			}
			if {$os == "windows"} {
				file copy -force $resultString $outfile
			} else {
				file link -symbolic $outfile $resultString
			}
		} 	
	}
}

# This methode parses an XML-File into a tcl-list. 
# This code was copied from the TCL-WiKi (http://wiki.tcl.tk/
 proc xml2list xml {
     regsub -all {>\s*<} [string trim $xml " \n\t<>"] "\} \{" xml
     set xml [string map {> "\} \{#text \{" < "\}\} \{"}  $xml]

     set res ""   ;# string to collect the result
     set stack {} ;# track open tags
     set rest {}
     foreach item "{$xml}" {
	## Ignore XML-comments
	 if {[string range $item 0 0] != "?"} {
		switch -regexp -- $item {
		^# {append res "{[lrange $item 0 end]} " ; #text item}
		^/ {
			regexp {/(.+)} $item -> tagname ;# end tag
			set expected [lindex $stack end]
			if {$tagname!=$expected} {error "$item != $expected"}
			set stack [lrange $stack 0 end-1]
			append res "\}\} "
		}
		/$ { # singleton - start and end in one <> group
		regexp {([^ ]+)( (.+))?/$} $item -> tagname - rest
		set rest [lrange [string map {= " "} $rest] 0 end]
		append res "{$tagname [list $rest] {}} "
		}
		default {
		set tagname [lindex $item 0] ;# start tag
		set rest [lrange [string map {= " "} $item] 1 end]
		lappend stack $tagname
		append res "\{$tagname [list $rest] \{"
		}
		}
		if {[llength $rest]%2} {error "att's not paired: $rest"}
		}
	}


     if [llength $stack] {error "unresolved: $stack"}
     string map {"\} \}" "\}\}"} [lindex $res 0]
 }


##################################################################################
##################################################################################
##################################################################################

# This script parses the XML-File specified by its first parameter for 
# a <ref>-Tag within a <file>-Tag. If one is found, a link pointing to the
# referenced file is created at the location specified by the second parameter. 
set infile ""
set outfile ""
set infile [lindex $argv 0]
set outfile [lindex $argv 1]
create_link_to_ref_tag_value $infile $outfile
exit
