#!/usr/bin/env wish

# helper function: get next line from MAP file, ignores comments and empty lines
proc ph_result_to_xmlph_nextline {infile} {
        set line ""
        while {($line == "") || ([string index $line 0] == "#")} {
                set line [string trim [gets $infile]]
        }
        return $line
}

# This method parses the EM result-file specified infile
# and calculates the MAP-description from it. Afterwards
# the MAP is written into the standard XML-format and saved
# at the location specified by outfile.)
proc ph_result_to_xmlph {infile_d0 infile_pi outfile} {
	set in_d0 [open $infile_d0]
	set in_pi [open $infile_pi]
	set d0 ""
	set d1 ""
	set pi ""
	# ignore first two lines of D0 and first line of pi
	set line [ph_result_to_xmlph_nextline $in_d0]
	set line [ph_result_to_xmlph_nextline $in_d0]
	set line [ph_result_to_xmlph_nextline $in_pi]
	# get matrix D0
	# get order
	set order [ph_result_to_xmlph_nextline $in_d0]
	# get rows for matrix D0
	array set D0row_entries {}
	for {set i 0} {$i < $order} {incr i} {
		set pi "$pi [ph_result_to_xmlph_nextline $in_pi]"
		# get columns
		set number_cols ""
		set diag ""
		while {$number_cols == ""} {
			set number_cols [ph_result_to_xmlph_nextline $in_d0]
			set diag [lindex $number_cols 1]
			set new_entry "\{[list $i $i]\} $diag"
			array set D0row_entries $new_entry
			set number_cols [lindex $number_cols 0]
		}
		for {set j 0} {$j < $number_cols} {incr j} {
			# get entries
			set entry [ph_result_to_xmlph_nextline $in_d0]
			set pos [lindex $entry 0]
			set value [lindex $entry 1]
			set new_entry "\{[list $i $pos]\} $value"
			array set D0row_entries $new_entry
		}
	}
	set tvec_entries ""
	# create string from the entries
	for {set i 0} {$i < $order} {incr i} {
		set D0line ""
		set tvec_entry 0
		for {set j 0} {$j < $order} {incr j} {
			if {[array get D0row_entries "[list $i $j]"] != [list]} {
				# file contained entry for element of D0
				set D0line "$D0line [lindex [array get D0row_entries [list $i $j]] 1]"
				set tvec_entry [expr $tvec_entry + [lindex [array get D0row_entries [list $i $j]] 1]]
			} else {
				# non-diagonal element => set to zero
				set D0line "$D0line 0"
			}
		}
		if {$d0 == ""} {
			set d0 $D0line
		} else {
			set d0 "$d0 \n$D0line"
		}
		set tvec_entries "$tvec_entries [expr -1.0 * $tvec_entry]"
	}
	for {set i 0} {$i < $order} {incr i} {
		set D1line ""
		for {set j 0} {$j < $order} {incr j} {
			set D1line "$D1line [expr [lindex $tvec_entries $i] * [lindex $pi $j]]"
		}
		if {$d1 == ""} {
			set d1 $D1line
		} else {
			set d1 "$d1 \n$D1line"
		}
	}
	# write PH file
	if {$outfile != ""} {
		set out_ph [open $outfile w+]
		puts $out_ph "<?xml version=\"1.0\"?>"
		puts $out_ph "<profido>"
		puts $out_ph "<ph>"
		puts $out_ph "<states>$order</states>"
		puts $out_ph "<d0>$d0"
		puts $out_ph "</d0>"
		puts $out_ph "<pi>$pi"
		puts $out_ph "</pi>"
		puts $out_ph "</ph>"
		puts $out_ph "<map>"
		puts $out_ph "<states>$order</states>"
		puts $out_ph "<d0>$d0"
		puts $out_ph "</d0>"
		puts $out_ph "<d1>$d1"
		puts $out_ph "</d1>"
		puts $out_ph "</map>"
		puts $out_ph "</profido>"
		close $out_ph
	}
	close $in_d0
	close $in_pi
}

##################################################################################
##################################################################################
##################################################################################

# This script parses the Momfit result-file specified by the first two parameters
# and calculates the PH-description from it. Afterwards the PH is written
# into the standard XML-format and saved at the location specified by the
# third parameter.
set infile1 ""
set infile2 ""
set outfile ""
set infile1 [lindex $argv 0]
set infile2 [lindex $argv 1]
set outfile [lindex $argv 2]
ph_result_to_xmlph $infile1 $infile2 $outfile
exit

