#!/bin/sh
# -----------------------------
# Export of workflow: batch
# -----------------------------

# This workflow was created with ProFiDo 1.2


# Before any further steps, check if all required external binaries are present in the path.
requirements="sh wish"
missingCommands=""
for curCmd in $requirements; do
	if ! [ `command -v $curCmd` ]; then
		missingCommands="$missingCommands\n$curCmd"
	fi
done
if ! [ -z "$missingCommands" ]; then
	echo "Warning, the following required external commands were not found on your system.\nPlease ensure their presence before executing the workflow.$missingCommands"
	exit 1
fi

# Before the actual execution, variables are set. To adapt the
# batch to another location just change the BASEDIR variable.
export BASEDIR="/home/gerloff/cvsshk/profido/rsc/export_samples"


# Derived folders: 
export RESULTDIR="${BASEDIR}/Results" 
export SOFTWAREDIR="${BASEDIR}/Software" 


# ----- Convert non-XML files ----- 

"${SOFTWAREDIR}/scripts/TraceToXMLTraceWithValueCount.sh" "${HOME}/ProFiDo/lbl3_dat.trace" "${RESULTDIR}/lbl3_dat.trace.xml"

# ----- Execute Jobs ----- 
# Execute Momfit
cd "${SOFTWAREDIR}/momfit/"

./momfit.sh "${RESULTDIR}/lbl3_dat.trace.xml" "${RESULTDIR}/batch_Momfit0.xml" 9 3 20000 1.0e-7 100000 13
# Execute JMomfit
cd "${SOFTWAREDIR}/jmomfit/"


./jmomfit.sh "${RESULTDIR}/lbl3_dat.trace.xml" "${RESULTDIR}/batch_Momfit0.xml" "${RESULTDIR}/MAP3_JM.xml" 5
# Execute ACfit
cd "${SOFTWAREDIR}/acfit/"


./acfit.sh "${RESULTDIR}/lbl3_dat.trace.xml" "${RESULTDIR}/batch_Momfit0.xml" "${RESULTDIR}/MAP3_AC.xml" 10
# Execute Print
cd "${SOFTWAREDIR}/Plot/"
"${SOFTWAREDIR}/scripts/CreateLinkToRefValueTarget.tcl" "${RESULTDIR}/lbl3_dat.trace.xml" "${RESULTDIR}/lbl3_dat.trace.xml.conv"
"${SOFTWAREDIR}/scripts/XMLMapToModgen.tcl" "${RESULTDIR}/MAP3_JM.xml" "${RESULTDIR}/MAP3_JM.xml.conv"
"${SOFTWAREDIR}/scripts/XMLMapToModgen.tcl" "${RESULTDIR}/MAP3_AC.xml" "${RESULTDIR}/MAP3_AC.xml.conv"
./plot.exe -format=latex -relative=false -start=1 -end=5 -plot=jmom -output="${RESULTDIR}/batch_Print3.xml.org" -trace "${RESULTDIR}/lbl3_dat.trace.xml.conv" -map "${RESULTDIR}/MAP3_JM.xml.conv" -map "${RESULTDIR}/MAP3_AC.xml.conv"
"${SOFTWAREDIR}/scripts/LatexToXMLLatex.sh" "${RESULTDIR}/batch_Print3.xml.org" "${RESULTDIR}/batch_Print3.xml"
# Execute Plot
cd "${SOFTWAREDIR}/Plot/"
"${SOFTWAREDIR}/scripts/CreateLinkToRefValueTarget.tcl" "${RESULTDIR}/lbl3_dat.trace.xml" "${RESULTDIR}/lbl3_dat.trace.xml.conv"
"${SOFTWAREDIR}/scripts/XMLMapToModgen.tcl" "${RESULTDIR}/MAP3_JM.xml" "${RESULTDIR}/MAP3_JM.xml.conv"
"${SOFTWAREDIR}/scripts/XMLMapToModgen.tcl" "${RESULTDIR}/MAP3_AC.xml" "${RESULTDIR}/MAP3_AC.xml.conv"
./plot.exe -format=ps -start=1 -end=10 -plot=ac -output="${RESULTDIR}/batch_Plot4.xml.org" -trace "${RESULTDIR}/lbl3_dat.trace.xml.conv" -map "${RESULTDIR}/MAP3_JM.xml.conv" -map "${RESULTDIR}/MAP3_AC.xml.conv"
"${SOFTWAREDIR}/scripts/ModgenPSToXMLPS.sh" "${RESULTDIR}/batch_Plot4.xml.org" "${RESULTDIR}/batch_Plot4.xml"


# ----- Convert XML-Files to external outputs ----- 

"${SOFTWAREDIR}/scripts/CreateCopyOffRefValueTarget.tcl" "${RESULTDIR}/batch_Print3.xml" "${HOME}/ProFiDo/lbl3_jmom.tex"
"${SOFTWAREDIR}/scripts/CreateCopyOffRefValueTarget.tcl" "${RESULTDIR}/batch_Plot4.xml" "${HOME}/ProFiDo/lbl3_ac.ps"